﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.DataAnalysis.Symbolic;

namespace HeuristicLab.Algorithms.DataAnalysis.Symbolic {
  [StorableClass]
  public sealed class SymbolicDataAnalysisIslandGAEvaluator<T> : SingleSuccessorOperator, IStochasticOperator
    where T : class,IDataAnalysisProblemData {
    private const string RandomParameterName = "Random";
    private const string EvaluatorParameterName = "Evaluator";
    private const string IslandNumberParameterName = "IslandNumber";
    private const string NumberOfIslandsParameterName = "NumberOfIslands";
    private const string ConstantSamplesParameterName = "ConstantSamples";
    private const string ConstantSamplesFitnessWeightFactorParameterName = "ConstantSamplesFitnessWeight";
    private const string RandomSamplesParameterName = "RandomSamples";
    private const string RandomSamplesFitnessWeightParameterName = "RandomSamplesFitnessWeight";
    private const string RandomSamplesConsecutiveParameterName = "RandomSamplesConsecutive";

    #region parameter properties
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters[RandomParameterName]; }
    }
    public IValueParameter<ISymbolicDataAnalysisSingleObjectiveEvaluator<T>> EvaluatorParameter {
      get { return (IValueParameter<ISymbolicDataAnalysisSingleObjectiveEvaluator<T>>)Parameters[EvaluatorParameterName]; }
    }

    public ILookupParameter<IntValue> NumberOfIslandParameter {
      get { return (ILookupParameter<IntValue>)Parameters[NumberOfIslandsParameterName]; }
    }
    public IFixedValueParameter<IntValue> ConstantSamplesParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters[ConstantSamplesParameterName]; }
    }
    public IFixedValueParameter<DoubleValue> ConstantSamplesFitnessWeightParameter {
      get { return (IFixedValueParameter<DoubleValue>)Parameters[ConstantSamplesFitnessWeightFactorParameterName]; }
    }
    public IFixedValueParameter<IntValue> RandomSamplesParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters[RandomSamplesParameterName]; }
    }
    public IFixedValueParameter<DoubleValue> RandomSamplesFitnessWeightParameter {
      get { return (IFixedValueParameter<DoubleValue>)Parameters[RandomSamplesFitnessWeightParameterName]; }
    }
    public IFixedValueParameter<BoolValue> RandomSamplesConsecutiveParameter {
      get { return (IFixedValueParameter<BoolValue>)Parameters[RandomSamplesConsecutiveParameterName]; }
    }
    #endregion

    #region properties
    public ISymbolicDataAnalysisSingleObjectiveEvaluator<T> Evaluator {
      get { return EvaluatorParameter.Value; }
      set { EvaluatorParameter.Value = value; }
    }
    public int ConstantSamples {
      get { return ConstantSamplesParameter.Value.Value; }
      set { ConstantSamplesParameter.Value.Value = value; }
    }
    public double ConstantSamplesFitnessWeight {
      get { return ConstantSamplesFitnessWeightParameter.Value.Value; }
      set { ConstantSamplesFitnessWeightParameter.Value.Value = value; }
    }
    public int RandomSamples {
      get { return RandomSamplesParameter.Value.Value; }
      set { RandomSamplesParameter.Value.Value = value; }
    }
    public double RandomSamplesFitnessWeight {
      get { return RandomSamplesFitnessWeightParameter.Value.Value; }
      set { RandomSamplesFitnessWeightParameter.Value.Value = value; }
    }
    public bool RandomSamplesConsecutive {
      get { return RandomSamplesConsecutiveParameter.Value.Value; }
      set { RandomSamplesConsecutiveParameter.Value.Value = value; }
    }
    #endregion

    [StorableConstructor]
    private SymbolicDataAnalysisIslandGAEvaluator(bool deserializing) : base(deserializing) { }
    private SymbolicDataAnalysisIslandGAEvaluator(SymbolicDataAnalysisIslandGAEvaluator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SymbolicDataAnalysisIslandGAEvaluator<T>(this, cloner);
    }

    public SymbolicDataAnalysisIslandGAEvaluator()
      : base() {

    }
  }
}
